/*  -*- c++ -*-
    
$Id: boundary.hpp 1511 2005-06-24 00:33:43Z yuk $
$Date: 2005-06-24 09:33:43 +0900 (Fri, 24 Jun 2005) $
$Rev: 1511 $
$Author: yuk $

*/

#ifndef BOUNDARY_HPP_ 
#define BOUNDARY_HPP_

#include <iostream>
#include "geometry.h"
#include "utils.h"

namespace BoundaryType
{
  enum btype{
    xyz_periodic = 0,
    xy_periodic = 1,
    x_periodic = 2,
    y_periodic = 3,
    z_periodic = 4,
    xyz_fixed = 5,
  };
}


class PeriodicBoundary
{
private:
  const double lx,ly,lz;
public:
  static const int type = BoundaryType::xyz_periodic; // boundary type 
  
  PeriodicBoundary(Geometry& geom, std::ostream& output = std::cerr):
    lx(geom.getX()),
    ly(geom.getY()),
    lz(geom.getZ())
    {

    }
  
  inline void boundarymove(Particles& ptcls){
    for(int i=0;i<ptcls.number;i++){
      if( ptcls.qx[i] < 0 ) {
        ptcls.qx[i] += lx;
      } else if (ptcls.qx[i] > lx ) {
        ptcls.qx[i] -= lx;
      }
      if( ptcls.qy[i] < 0 ) {
        ptcls.qy[i] += ly;
      } else if (ptcls.qy[i] > ly ) {
        ptcls.qy[i] -= ly;
      }
      if( ptcls.qz[i] < 0 ) {
        ptcls.qz[i] += lz;
      } else if (ptcls.qz[i] > lz ) {
        ptcls.qz[i] -= lz;
      }
    }
  }
  
  inline void canonicalize(double& dx, double& dy, double& dz){
    if(dx > lx/2) dx -= lx;
    if(dy > ly/2) dy -= ly;
    if(dz > lz/2) dz -= lz;
    if(dx < -lx/2) dx += lx;
    if(dy < -ly/2) dy += ly;
    if(dz < -lz/2) dz += lz;
  }
  
  inline void boundaryforce(Particles& ptcls){}
  
  inline int getBoundaryType(void){
    return type;
  }
  inline double boundaryEnergy(void){
    return 0;
  }
  
};

//========================================================


#endif // BOUNDARY_HPP_ 
