/*  -*- c++ -*-
  $Id: mesh.h 1511 2005-06-24 00:33:43Z yuk $
  $Date: 2005-06-24 09:33:43 +0900 (Fri, 24 Jun 2005) $
  $Rev: 1511 $
  $Author: yuk $

  
*/

#ifndef MESH_H_
#define MESH_H_

#include <iostream>
#include <cmath>
#include "utils.h"
#include "boundary.hpp"
#include "dist.h"

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


#define THROWERROR 1

class MeshOverFlow
{
 private:
   int ia,mm;
 public:
   MeshOverFlow(int iaddr, int maxnum){
      ia =  iaddr;
      mm = maxnum;
   }
   void printer(std::ostream& output = std::cerr){
      output << "Mesh Over Flow: addr " << ia 
		<< " max num." << mm<< std::endl;
   }
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


class Mesh
{
//---------------------------------------------------
  private:

   // linear world size
   int msx;
   int msy;
   int msz;

   // actual linear size
   // with one margin
   int lxm;
   int lym;
   int lzm;
   
   // actual linear size
   // with two margin
   int lxm2;
   int lym2;
   int lzm2;

   // actual memory size
   int meshdim;

//   static const int maxoccnum; 
   enum {maxoccnum = 200};
   
   double lenx;
   double leny;
   double lenz;

   double meshsize2;
   
   inline void clearmesh(void){
      for(int i=0;i<meshdim;i++){
	 numlist[i] = 0;
      }
   }
//---------------------------------------------------   
   inline void regist(int imesh,int iparticle){
      int idx = numlist[imesh];
#ifdef THROWERROR
      if(idx == maxoccnum ){
	 throw MeshOverFlow(imesh,idx);
      } else {
#endif // THROWERROR
	 meshlist[imesh][idx] = iparticle;
	 numlist[imesh] += 1;
#ifdef THROWERROR
      }
#endif // THROWERROR
   }
//---------------------------------------------------
   void registcheck(std::ostream& output = std::cerr){
      for(int i=0;i<meshdim;i++){
	 output << "mesh regist addr: "  << i <<':';
	 output << " ix: " << iadrtoindex(i)[0] << ' ' ;
	 output << " iy: " << iadrtoindex(i)[1] << ' ' ;
	 output << " iz: " << iadrtoindex(i)[2] << ' ' ;
	 if(numlist[i] == maxoccnum){
	    throw MeshOverFlow(i,maxoccnum);
	 } else {
	    for(int j=0;j<numlist[i];j++){
	       output <<meshlist[i][j] << ' ' ;
	    }
	    output << '\n';
	 }
      }
   }
//----------------------------------------------------
   
 public:   
   static const int slicecnt = 4;

//   static const int maxoccnum = 200; 
//   static const int slicecnt;

   const int meshsize;
   int **meshlist;
   int *numlist;
   // int ipadr[14];
   int ipadr[27];
//--------------------------------------------------------   
   Mesh(Geometry& geom, int msize, std::ostream& output = std::cerr):
      meshsize(msize){
      
      lenx = geom.getX();
      leny = geom.getY();
      lenz = geom.getZ();
      
      msx = static_cast<int>(lenx/meshsize);
      msy = static_cast<int>(leny/meshsize);
      msz = static_cast<int>(lenz/meshsize);
      
      meshsize2 = meshsize*meshsize;

/*      
      info<int>("Mesh","mesh size",meshsize,output);
      
      info<int>("Mesh","mesh size x",msx,output);
      info<int>("Mesh","mesh size y",msy,output);
      info<int>("Mesh","mesh size z",msz,output);
      
      info<int>("Mesh","Observation slice number:",slicecnt*msz,output);
      info<double>("Mesh","Observation interval:",
                   static_cast<double>(meshsize)/static_cast<double>(slicecnt)
                   ,output);
*/    
      
      lxm = msx + 1;
      lym = msy + 1;
      lzm = msz + 1;
      
      lxm2 = msx+2;
      lym2 = msy+2;
      lzm2 = msz+2;
      
      meshdim = lxm2*lym2*lzm2;
      
      if( std::fabs(lenx-static_cast<double>(msx*meshsize))> 1e-5 )
      {
         error("Mesh"," meshsize mismatch",output);
         error<int>("Mesh","X mesh size ", msx,output);
         error<double>("Mesh","  real size ", lenx,output);
         error<int>("Mesh","  mesh size ", meshsize,output);
         std::abort();
      }
      if( std::fabs(leny-static_cast<double>(msy*meshsize))> 1e-5 )
      {
         error("Mesh"," meshsize mismatch",output);
         error<int>("Mesh","Y mesh size ", msy,output);
         error<double>("Mesh","  real size ", leny,output);
         error<int>("Mesh","  mesh size ", meshsize,output);
         std::abort();
      }
      if( std::fabs(lenz-static_cast<double>(msz*meshsize))> 1e-5 )
      {
         error("Mesh"," meshsize mismatch",output);
         error<int>("Mesh","Z mesh size ", msz,output);
         error<double>("Mesh","  real size ", lenz,output);
         error<int>("Mesh","  mesh size ", meshsize,output);
         std::abort();
      }
      
      meshlist = new int *[meshdim];
      for(int i=0;i<meshdim;i++){
         meshlist[i] = new int[maxoccnum];
      }
      
      numlist = new int[meshdim];
      for(int i=0;i<meshdim;i++){
         numlist[i] = 0;
      }
      
      for(int i=0;i<meshdim;i++){
         for(int j=0;j<maxoccnum;j++){
            meshlist[i][j] = 0;
         }
      }
      // set relative addr vector
      
      ipadr[0]  = index(0,0,0)-index(0,0,0);
      ipadr[1]  = index(1,0,0)-index(0,0,0);
      ipadr[2]  = index(0,1,0)-index(0,0,0);
      ipadr[3]  = index(1,1,0)-index(0,0,0);
      ipadr[4]  = index(1,-1,1)-index(0,0,0);
      ipadr[5]  = index(0,0,1)-index(0,0,0);
      ipadr[6]  = index(1,0,1)-index(0,0,0);
      ipadr[7]  = index(0,1,1)-index(0,0,0);
      ipadr[8]  = index(1,1,1)-index(0,0,0);
      ipadr[9]  = index(1,-1,0)-index(0,0,0);
      ipadr[10] = index(1,1,-1)-index(0,0,0);
      ipadr[11] = index(0,1,-1)-index(0,0,0);
      ipadr[12] = index(1,0,-1)-index(0,0,0);
      ipadr[13] = index(1,-1,-1)-index(0,0,0);
      
      // doesn't use bulk calculation 
      // only for convenient
      
      ipadr[14] = index(-1,1,-1)-index(0,0,0);
      ipadr[15] = index(-1,0,-1)-index(0,0,0);
      ipadr[16] = index(-1,-1,-1)-index(0,0,0);
      ipadr[17] = index(0,0,-1)-index(0,0,0);
      ipadr[18] = index(0,-1,-1)-index(0,0,0);
      
      ipadr[19] = index(-1,1,0)-index(0,0,0);
      ipadr[20] = index(-1,0,0)-index(0,0,0);
      ipadr[21] = index(-1,-1,0)-index(0,0,0);
      ipadr[22] = index(0,-1,0)-index(0,0,0);
      
      ipadr[23] = index(-1,1,1)-index(0,0,0);
      ipadr[24] = index(-1,0,1)-index(0,0,0);
      ipadr[25] = index(-1,-1,1)-index(0,0,0);
      ipadr[26] = index(0,-1,1)-index(0,0,0);
      
   }
//---------------------------------------------------
   ~Mesh(void){
      for(int i=0;i<meshdim;i++){
	 delete[] meshlist[i] ;
      }
      delete[] meshlist;
      delete[] numlist;
   }
//---------------------------------------------------   
   inline int index(int ix,int iy,int iz){
      return (lxm2*lym2+lxm2+1)+ ix + lxm2 * iy + lym2*lxm2*iz;
   }
//---------------------------------------------------
   inline int* iadrtoindex(int iadr){
      int i = iadr-lxm2*lym2-lxm2-1;
      int ix = i%lxm2;
      ix = (ix < -1 ? ix + lxm2 :ix);
      ix = (ix > msx ? ix - lxm2:ix);
      
      i -= ix;
      i /= lxm2;
      int iy = i%lym2;
      iy = (iy < -1 ? iy + lym2 :iy);
      iy = (iy > msy ? iy - lym2:iy);
      
      i -= iy;
      int iz = i/lym2;
      iz = (iz < -1 ? iz + lzm2 :iz);
      iz = (iz > msz ? iz - lzm2:iz);

      int* a = new int[3];

      a[0] = ix;
      a[1] = iy;
      a[2] = iz;
      return a;
   } // iadrtoindex 
//--------------------------------------------------------
   inline Mesh& registParticles(Particles& ptcl){
      clearmesh();
#ifdef THROWERROR
      try{
#endif // THROWERROR
	 for(int i=0;i<ptcl.number;i++){

               int ix = static_cast<int>(ptcl.qx[i]/meshsize);
               int iy = static_cast<int>(ptcl.qy[i]/meshsize);
               int iz = static_cast<int>(ptcl.qz[i]/meshsize);
               int idx = index(ix,iy,iz);
               regist(idx,i);

	 }
#ifdef THROWERROR
      }
      catch(MeshOverFlow x){
	 x.printer();
	 std::abort();
      }
#endif // THROWERROR      
      
      
      return *this;
   }

//------------------------------------------------------------   
   inline Mesh& additionalregistParticle(double q[],int idx){
#ifdef THROWERROR
      try{
#endif // THROWERROR
         int ix = static_cast<int>(q[0]/meshsize);
         int iy = static_cast<int>(q[1]/meshsize);
         int iz = static_cast<int>(q[2]/meshsize);
         
         int iad = index(ix,iy,iz);
         regist(iad,idx);
      }
#ifdef THROWERROR
      catch(MeshOverFlow x){
         x.printer();
         std::abort();
      }
#endif // THROWERROR     
 
       return *this;
   }
//---------------------------------------------------   
   template <class Boundary>
   Mesh& generatePtable(Particles& ptcl,PairTable& ptbl,Boundary& bndry){

      registParticles(ptcl);

      /*
	index(x,y,z)
	simulation box     
	x \in [0,1,..,msx-1]
	y \in [0,1,..,msy-1]
	z \in [0,1,..,msz-1]
	boundary
	x \in [-1,msx]
	y \in [-1,msy]
	z \in [-1,msz]
      */

      if(bndry.type == BoundaryType::xyz_periodic){ // periodic boundary
     	 copyboundaryXYZ();
      } else if (bndry.type == BoundaryType::xy_periodic) { // two plate boundary
         copyboundaryXY();
      }

      // clear pair table parameters
      
      int l = 0;
      
      // for in system onbox
      for(int iii=0;iii<msx*msy*msz;iii++){
	 // create local addr
	 
	 int ix = iii%msx;
	 int iy = (iii - ix)/msx;
	 iy %= msy;
	 int iz = (iii-ix-iy*msx)/msy/msx;
	 int iadr = index(ix,iy,iz);
	 int inum = numlist[iadr];


	 // registしたばっかりなので、メッシュには完全に詰まっている。
	 for(int i=0;i<inum;i++){
	    int ip1 = meshlist[iadr][i];
	    for(int j=i+1;j<inum;j++){
	       int ip2 = meshlist[iadr][j];
               
	       double dxt = ptcl.qx[ip1]-ptcl.qx[ip2];
	       double dyt = ptcl.qy[ip1]-ptcl.qy[ip2];
	       double dzt = ptcl.qz[ip1]-ptcl.qz[ip2];

	       bndry.canonicalize(dxt,dyt,dzt);

	       double dtmp = dxt*dxt+dyt*dyt+dzt*dzt;

	       if(dtmp < meshsize2) {
		  ptbl.dist[l] = dtmp;
		  ptbl.idx1[l]=ip1;
		  ptbl.idx2[l]=ip2;
		  l++;
	       }
	    }
	 }
      } // on-box loop
      

      // for in system ipadr[1-13]
      for(int jjj=1;jjj<=13;jjj++){
	 for(int iii=0;iii<msx*msy*msz;iii++){
	    // create local addr
	    int ix = iii%msx;
	    int iy = (iii - ix)/msx;
	    iy %= msy;
	    int iz = (iii-ix-iy*msx)/msy/msx;
	    
	    int iadr = index(ix,iy,iz);
	    int iadr2 = iadr + ipadr[jjj];
	    
	    int inum1 = numlist[iadr];
	    int inum2 = numlist[iadr2];
	    
	    if(inum1 != 0 && inum2 != 0){
	       for(int i=0;i<inum1;i++){
		  int ip1 = meshlist[iadr][i];
		  for(int j=0;j<inum2;j++){
		     int ip2 = meshlist[iadr2][j];
                     
		     double dxt = ptcl.qx[ip1]-ptcl.qx[ip2];
		     double dyt = ptcl.qy[ip1]-ptcl.qy[ip2];
		     double dzt = ptcl.qz[ip1]-ptcl.qz[ip2];

		     bndry.canonicalize(dxt,dyt,dzt);
		     
		     double dtmp = dxt*dxt+dyt*dyt+dzt*dzt;
		     
		     if(dtmp < meshsize2) {
			ptbl.dist[l] = dtmp;
			ptbl.idx1[l]=ip1;
			ptbl.idx2[l]=ip2;
			l++;
		     }
		  }
	       }
	    }
	 }
      }


      ptbl.pairnum = l;
      return *this;
   } // generatePtable
//---------------------------------------------------   
   template <class Boundary>
   Mesh& updatePtable(Particles& ptcl,PairTable& ptbl, Boundary& bndry){
      for(int l=0;l<ptbl.pairnum;l++){

	 int i=ptbl.idx1[l];
	 int j=ptbl.idx2[l];
         

            double dxt = ptcl.qx[i]-ptcl.qx[j];
            double dyt = ptcl.qy[i]-ptcl.qy[j];
            double dzt = ptcl.qz[i]-ptcl.qz[j];

            bndry.canonicalize(dxt,dyt,dzt);

            double dtmp = dxt*dxt+dyt*dyt+dzt*dzt;
            
            ptbl.dist[l] = dtmp;
         
      }
      return *this;
   }
//---------------------------------------------------      
   template <class PC, class PT, class Boundary>
   Mesh& generatePtableSimple(PC& ptcl, PT& ptbl, Boundary& bndry){
      int l=0;
      for(int i=0;i<ptcl.number;i++){
	 for(int j=i+1;j<ptcl.number;j++){

               double dxt = ptcl.qx[i]-ptcl.qx[j];
               double dyt = ptcl.qy[i]-ptcl.qy[j];
               double dzt = ptcl.qz[i]-ptcl.qz[j];
               
               bndry.canonicalize(dxt,dyt,dzt);
               
               double dtmp = dxt*dxt+dyt*dyt+dzt*dzt;
               
               if(dtmp < meshsize2) {
                  ptbl.dist[l] = dtmp;
                  ptbl.idx1[l] = i;
                  ptbl.idx2[l] = j;
                  l++;

            }
         }
      }
      ptbl.pairnum = l;
      return *this;
   }
//---------------------------------------------------   
   inline int getX(){
      return msx;
   }
//---------------------------------------------------   
   inline int getY(){
      return msy;
   }
//---------------------------------------------------   
   inline int getZ(){
      return msz;
   }
//---------------------------------------------------      
   inline int getMsize(){
      return meshsize;
   }
//---------------------------------------------------   
inline double getXlen(void) const{
      return lenx;
   }
//---------------------------------------------------   
   inline double getYlen(void) const{
      return leny;
   }
//---------------------------------------------------   
   void copyboundaryXYZ(void){
      /*
	index(x,y,z)
	
	simulation box     
	x \in [0,1,..,msx-1]
	y \in [0,1,..,msy-1]
	z \in [0,1,..,msz-1]
	
	boundary
	x \in [-1,msx]
	y \in [-1,msy]
	z \in [-1,msz]
      */
      {
	 // periodic boundary
	 
	 // x[0] to x[msx]
	 int i=0;
	 for(int j=0;j<msy;j++){
	    for(int k=0;k<msz;k++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(msx,j,k);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
      {
	 // x[msx-1] to x[-1]
	 int i=msx-1;
	 for(int j=0;j<msy;j++){
	    for(int k=0;k<msz;k++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(-1,j,k);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
      {
	 // y[0] to y[msy]
	 int j=0;
	 for(int i=-1;i<lxm;i++){
	    for(int k=0;k<msz;k++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(i,msy,k);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
      {
	 // y[msy-1] to y[-1]
	 int j=msy-1;
	 for(int i=-1;i<lxm;i++){
	    for(int k=0;k<msz;k++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(i,-1,k);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
      {
	 // z[0] to z[msz]
	 int k=0;
	 for(int i=-1;i<lxm;i++){
	    for(int j=-1;j<lym;j++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(i,j,msz);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
      {
	 // z[msz-1] to z[-1]
	 int k=msz-1;
	 for(int i=-1;i<lxm;i++){
	    for(int j=-1;j<lym;j++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(i,j,-1);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
   }
//---------------------------------------------------
   void copyboundaryXYfor(double q[],int pindx){
      // 追加した粒子が境界にいれば、この粒子が入っている場所のみコピー
      // 一粒子だけ追加されたときにしか呼んではいけない。
      int ix = static_cast<int>(q[0]/meshsize);
      int iy = static_cast<int>(q[1]/meshsize);
      int iz = static_cast<int>(q[2]/meshsize);

      if(ix == 0 && ( iy != 0  || iy != msy-1)){
         int iadr  = index(0,iy,iz);
         int iadr2 = index(msx,iy,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
      } else if (ix == msx-1 && (iy != 0  || iy != msy-1)) {
         int iadr  = index(msx-1,iy,iz);
         int iadr2 = index(-1,iy,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
      } else if (iy == 0 && (ix != 0 || ix != msx-1)) {
         int iadr  = index(ix,0,iz);
         int iadr2 = index(ix,msy,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
      } else if (iy == msy-1 && (ix != 0 || ix != msx-1)) {
         int iadr  = index(ix,msy-1,iz);
         int iadr2 = index(ix,-1,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
      } else if (ix ==0 && iy == 0) {
         int iadr  = index(0,0,iz);
         int iadr2 = index(msx,0,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
         
         iadr2 = index(0,msy,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];

         iadr2 = index(msx,msy,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
      } else if (ix ==0 && iy == msy-1) {
         int iadr  = index(0,msy-1,iz);
         
         int iadr2 = index(msx,msy-1,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
         
         iadr2 = index(0,-1,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];

         iadr2 = index(msx,-1,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
      } else if (ix ==msx-1 && iy == 0) {
         int iadr  = index(msx-1,0,iz);
         
         int iadr2 = index(-1,0,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
         
         iadr2 = index(msx-1,msy,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];

         iadr2 = index(-1,msy,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
      } else if (ix ==msx-1 && iy == msy-1) {
         int iadr  = index(msx-1,msy-1,iz);
         
         int iadr2 = index(-1,msy-1,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
         
         iadr2 = index(msx-1,-1,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];

         iadr2 = index(-1,-1,iz);
         meshlist[iadr2][numlist[iadr]-1] = meshlist[iadr][numlist[iadr]-1];
         numlist[iadr2] = numlist[iadr];
      } 
      
      
      
      
   }
//---------------------------------------------------   
   void copyboundaryXY(void){
      /*
	index(x,y,z)
	
	simulation box     
	x \in [0,1,..,msx-1]
	y \in [0,1,..,msy-1]
	z \in [0,1,..,msz-1]
	
	boundary
	x \in [-1,msx]
	y \in [-1,msy]
	z \in [-1,msz]
      */
      {
	 // periodic boundary
	 
	 // x[0] to x[msx]
	 int i=0;
	 for(int j=0;j<msy;j++){
	    for(int k=0;k<msz;k++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(msx,j,k);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
      {
	 // x[msx-1] to x[-1]
	 int i=msx-1;
	 for(int j=0;j<msy;j++){
	    for(int k=0;k<msz;k++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(-1,j,k);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
      {
	 // y[0] to y[msy]
	 int j=0;
	 for(int i=-1;i<lxm;i++){
	    for(int k=0;k<msz;k++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(i,msy,k);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
      {
	 // y[msy-1] to y[-1]
	 int j=msy-1;
	 for(int i=-1;i<lxm;i++){
	    for(int k=0;k<msz;k++){
	       int iadr  = index(i,j,k);
	       int iadr2 = index(i,-1,k);
	       for(int l=0;l<numlist[iadr];l++){
		  meshlist[iadr2][l] = meshlist[iadr][l];
	       }
	       numlist[iadr2] = numlist[iadr];
	    }
	 }
      }
   }
//---------------------------------------------------   

   
      
};

#endif // MESH_H_


