/*
  -*- c++ -*-
  $Id: mtrand.cpp 1511 2005-06-24 00:33:43Z yuk $
  $Date: 2005-06-24 09:33:43 +0900 (Fri, 24 Jun 2005) $
  $Rev: 1511 $
  $Author: yuk $
 
*/

#include "mtrand.h"
#include <cmath>

mtrand::mtrand(void)
{
   unsigned long s = 123456789UL;
   initialseed = s;
   state[0]= s & 0xffffffffUL;
   for (int j=1; j<N; j++) {
      state[j] = (1812433253UL * (state[j-1] ^ (state[j-1] >> 30)) + j); 
      /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
      /* In the previous versions, MSBs of the seed affect   */
      /* only MSBs of the array state[].                        */
      /* 2002/01/09 modified by Makoto Matsumoto             */
      state[j] &= 0xffffffffUL;  /* for >32 bit machines */
   }
   left = 1; initf = 1;
}

mtrand::mtrand(unsigned long s)
{
   state[0]= s & 0xffffffffUL;
   initialseed = s;
   for (int j=1; j<N; j++) {
      state[j] = (1812433253UL * (state[j-1] ^ (state[j-1] >> 30)) + j); 
      /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
      /* In the previous versions, MSBs of the seed affect   */
      /* only MSBs of the array state[].                        */
      /* 2002/01/09 modified by Makoto Matsumoto             */
      state[j] &= 0xffffffffUL;  /* for >32 bit machines */
   }
   left = 1; initf = 1;
}

mtrand::~mtrand(void){}

void mtrand::next_state(void)
{
   unsigned long *p=state;
   int j;
   
   left = N;
   next = state;
   
   for (j=N-M+1; --j; p++) 
      *p = p[M] ^ twist(p[0], p[1]);
   
   for (j=M; --j; p++) 
      *p = p[M-N] ^ twist(p[0], p[1]);

   *p = p[M-N] ^ twist(p[0], state[0]);
}

unsigned long mtrand::genrand_int32(void)
{
   unsigned long y;
   
   if (--left == 0) next_state();
   y = *next++;
   
   /* Tempering */
   y ^= (y >> 11);
   y ^= (y << 7) & 0x9d2c5680UL;
   y ^= (y << 15) & 0xefc60000UL;
   y ^= (y >> 18);
   
   return y;
}


// include 1
double mtrand::genrand_real01(void){
   unsigned long y;
   
   if(--left==0) next_state();
   y = *next++;
   
   /* Tempering */
   y ^= (y >> 11);
   y ^= (y << 7) & 0x9d2c5680UL;
   y ^= (y << 15) & 0xefc60000UL;
   y ^= (y >> 18);
   
   return (double) y * (1.0/4294967295.0); 
   
}


// not include 1
double mtrand::genrand_real01b(void){
   unsigned long y;
   
   if(--left==0) next_state();
   y = *next++;
   
   /* Tempering */
   y ^= (y >> 11);
   y ^= (y << 7) & 0x9d2c5680UL;
   y ^= (y << 15) & 0xefc60000UL;
   y ^= (y >> 18);
   
   return (double) y * (1.0/4294967296.0); 
   
}

// normal distriubtion
// default: iteration number is fixed
// sigma = 1, mean = 0
double mtrand::genrand_nd(void){
   unsigned long y;
   double xx;
   
   int counter = 20;
   
   xx = 0.0;
   
   for(int i = 0; i < counter;i++){
      
      if(--left==0) next_state();
      y = *next++;
      /* Tempering */
      y ^= (y >> 11);
      y ^= (y << 7) & 0x9d2c5680UL;
      y ^= (y << 15) & 0xefc60000UL;
      y ^= (y >> 18);
      
      xx +=  (double) y * (1.0/4294967295.0); 
   }
   
   xx -= 0.5*(double) counter;
   
   return xx * std::sqrt(12.0/(double) counter);
   
}



// normal distriubtion
// default: iteration number is fixed
// sigma = 1, mean = 0
double mtrand::genrand_nd(double sigma){
   unsigned long y;
   double xx;
   
   int counter = 20;
   
   xx = 0.0;
   
   for(int i = 0; i < counter;i++){
      
      if(--left==0) next_state();
      y = *next++;
      /* Tempering */
      y ^= (y >> 11);
      y ^= (y << 7) & 0x9d2c5680UL;
      y ^= (y << 15) & 0xefc60000UL;
      y ^= (y >> 18);
      
      xx +=  (double) y * (1.0/4294967295.0); 
   }
   
   xx -= 0.5*(double) counter;
   
   return xx * std::sqrt(12.0*sigma/(double) counter);
   
}


