/*  -*- c++ -*-
    $Id: particlemodels.hpp 1512 2005-06-24 00:57:06Z yuk $
    $Date: 2005-06-24 09:57:06 +0900 (Fri, 24 Jun 2005) $
    $Rev: 1512 $
    $Author: yuk $
    */

#ifndef PARTICLEMODELS_HPP_
#define PARTICLEMODELS_HPP_

#include <iostream>
#include "utils.h"

class LennardJones 
{
 private:
   const double epsilon;
   const double sigma;

   double sealing;
   const double sigma2;
   
 public:
   const double cutoff;
   const double cutoff2;
   
   LennardJones(double tcutoff,std::ostream& output= std::cerr,
		double tepsilon=1.0,double tsigma = 1.0):
      epsilon(tepsilon),
      sigma(tsigma),
      sigma2(tsigma*tsigma),
      cutoff(tcutoff),
      cutoff2(tcutoff*tcutoff)
      {
	 
	 double sdd6 = sigma2/cutoff2;
	 sdd6 = sdd6*sdd6*sdd6;
	 sealing = 4.0*epsilon*(sdd6-1.0)*sdd6;
   };
   
   inline double pairPotential(double dist2) const {
      double dd6 = sigma2/dist2;
      dd6 = dd6*dd6*dd6;
      
      return (dist2 < cutoff2
	      ? 4.0*epsilon*dd6*(dd6-1.0)-sealing: 0
	      );
   }
    
   inline double force(double dist2) const{
      double dd6=sigma2/dist2;
      double dfac= 24.0*epsilon*dd6;
      dd6=dd6*dd6*dd6;
      return (dist2 < cutoff2
	      ? dfac*(2.0*dd6-1.0)*dd6:0);
   }
   
};



#endif //PATICLEMODELS_HPP_

