/*  -*- c++ -*-
  $Id: utils.cpp 1511 2005-06-24 00:33:43Z yuk $
  $Date: 2005-06-24 09:33:43 +0900 (Fri, 24 Jun 2005) $
  $Rev: 1511 $
  $Author: yuk $
  */

#include <iostream>
#include <cmath>
#include "utils.h"

void error(char* cname, char* outstr,std::ostream& output) {
   output <<  cname << ": error:" << outstr 
	  << ' ' << std::endl;
}


void info(char* cname,char* outstr,std::ostream& output) {
   output << "# "<< cname << ": info:"<< outstr 
	  << ' ' << std::endl;
}


void pack(int num,int offset,double qx[] ,double qy[],double qz[],
	  double e0[],
	  double e1[],double e2[],double e3[],
	  double x1,double x2,double y1,
	  double y2,double z1,double z2,std::ostream& output)
{
/*
  pack particles with base vectors e1,e2,e3 in a box [x2-x1,y2-y1,z2-z1].
  An origin is given by e0.
*/
   
//   const int len = 6;
   const int len = 40;
   const int dlen = len*2+1;
   
   double* tqx;
   double* tqy;
   double* tqz;
   
   tqx = new double[dlen*dlen*dlen];
   tqy = new double[dlen*dlen*dlen];
   tqz = new double[dlen*dlen*dlen];
   
   int l = 0;
   int irem = num;
   
   for(int ic = -dlen;ic<= 3*dlen;ic+=dlen){
      for(int jc = -dlen;jc<= 3*dlen;jc+=dlen){
	 for(int kc = -dlen;kc<= 3*dlen;kc+=dlen){
	    // generate an integer grid [-len,len]x[-len,len]x[-len,len]
	    int ll = 0;
	    for(int m=0;m<dlen*dlen*dlen;m++){
	       int it1 = m/dlen;
	       int it2 = m/(dlen*dlen);
	       int it3 = m-it1*dlen;
	       int it4 = it1/dlen;
	       int it5 = it1-it4*dlen;
	       
	       int i = it3-len;
	       int j = it5-len;
	       int k = it2-len;
               //std::cout << i+ic << ' ' << j+jc << ' ' << k+kc << '\n';
	       double rx = (i+ic)*e1[0]+(j+jc)*e2[0]+(k+kc)*e3[0]+e0[0]+x1;
	       double ry = (i+ic)*e1[1]+(j+jc)*e2[1]+(k+kc)*e3[1]+e0[1]+y1;
	       double rz = (i+ic)*e1[2]+(j+jc)*e2[2]+(k+kc)*e3[2]+e0[2]+z1;
	       if(rx >= x1 && rx <= x2){
		  if(ry >= y1 && ry <= y2){
		     if(rz >= z1 && rz <= z2){
			tqx[ll] = rx;
			tqy[ll] = ry;
			tqz[ll] = rz;
			ll ++;
		     }
		  }
	       }
	    }
	    // tentative pack 
	    int icnt = (ll<=irem?ll:irem);
	    for(int i=0;i<icnt;i++){
	       qx[i+l+offset] = tqx[i];
	       qy[i+l+offset] = tqy[i];
	       qz[i+l+offset] = tqz[i];
	    }
	    l += icnt;
	    irem = irem-icnt;
	    if(icnt != 0) {
              /*
	       output << "#packing: num:"<< num 
		      << " pack:" << l << " remain:"
		      << irem << " " << ic*e1[0]+jc*e2[0]+kc*e3[0]+e0[0]+x1 
                      << " " << ic*e1[1]+jc*e2[1]+kc*e3[1]+e0[1]+y1 
                      << " " << ic*e1[2]+jc*e2[2]+kc*e3[2]+e0[2]+z1 << "\n";
              */
	    }
	    
            
	    if(l == num){
	       delete[] tqx;
	       delete[] tqy;
	       delete[] tqz;
               
               output.flush();
               
	       return;
	    }
	 }
      }
   }
   
   output << "#packing: num:"<< num 
	  << " pack:" << l << " remain:"
	  << irem << '\n';
   output << "Error: Packing routine: cannot pack" << std::endl;
   output.flush();
   
   std::abort();
   
}

//-------------------------------------------------------
unsigned long mkseeds(unsigned long seed,int myid,std::ostream& output)
{
  unsigned long iy[500];
  
  // 32bit masking
  iy[0] = seed & 0xffffffff; 
//  info<unsigned long>("Make Seeds","iy[0]:",iy[0],output);
  
  for( int i=1;i<31;i++){
    iy[i] = 1812433253UL * (iy[i-1] ^ (iy[i-1] >>30 )) + i;
    iy[i] &= 0xffffffff;   // 32bit masking
//    info<int,unsigned long>("Make Seeds","iy:[","]:",i,iy[i],output);
  }
  // M random number
  for(int i=31;i<500;i++){
    iy[i] = iy[i-31] ^ iy[i-13];
//    info<int,unsigned long>("Make Seeds","iy:[","]:",i,iy[i],output);
  }
  
  return iy[499-myid];
  
}


